-- Portions of this code were contributed by Martin Ashwill, Lukas Price, Ryan Young, and Harry Lewis
-- You may modify this script for personal use, but please do not release said modifications without prior permission
-- Matt Jauchler, 2022

require ("Assets/RSC/P32Pack01/RailVehicles/Diesel/P32DC/CommonScripts/P32DC_EngineScript.out");

local oldInit = Initialise
local oldUpdate = Update
local oldOnCvChange = OnControlValueChange
local flashstart_s = nil

function Initialise()
    oldInit()
    Call ("BeginUpdate")
end


function OnControlValueChange ( name, index, value )
	-- Preserve the old CV change handler; it's needed for the alerter and other
	-- functions.
	oldOnCvChange(name, index, value)

	if name == "Headlights" then
		if value == 0 then
			Call( "FH1:Activate", 0 )
			Call( "FH2:Activate", 0 )
			Call( "BH1:Activate", 0 )
			Call( "BH2:Activate", 0 )
			Call("Childs_NEW:ActivateNode", "headlightcovers", 0)
		elseif value == 1 then
			Call( "FH1:Activate", 1 )
			Call( "FH2:Activate", 1 )
			Call( "BH1:Activate", 0 )
			Call( "BH2:Activate", 0 )
			Call("Childs_NEW:ActivateNode", "headlightcovers", 1)
		elseif value == 2 then
			Call( "FH1:Activate", 0 )
			Call( "FH2:Activate", 0 )
			Call( "BH1:Activate", 1 )
			Call( "BH2:Activate", 1 )
			Call("Childs_NEW:ActivateNode", "headlightcovers", 0)
		end   
	end
end


local function SetDitchLights()
	local now_s = Call("GetSimulationTime")
	local headlights = Call("GetControlValue", "Headlights", 0)
	local ditchlights = Call("GetControlValue", "Ditchlights", 0)

	local flashtime_s = 10
	local startflash = headlights == 1 and Call("GetControlValue", "Horn", 0) > 0
	-- flashing just began; record the start time
	if startflash then
		flashstart_s = now_s
	-- flashing period elapsed
	elseif flashstart_s ~= nil and now_s - flashstart_s > flashtime_s then
		flashstart_s = nil
	end

	local cycle_s = 1.7
	local flashing = flashstart_s ~= nil and now_s - flashstart_s <= flashtime_s
	local fixed = ditchlights == 1 and not flashing
	-- modulus of the time since flashing began determines whether to show the
	-- left or right headlight
	local flashleft = flashstart_s ~= nil
		and math.mod(now_s - flashstart_s, cycle_s) < cycle_s / 2
	-- finally, turn the headlights on and off
	local showleft = (fixed or (flashing and flashleft)) and 1 or 0
	Call("ActivateNode", "ditch_left", showleft)
	Call("DLL:Activate", showleft)
	local showright = (fixed or (flashing and not flashleft)) and 1 or 0
	Call("ActivateNode", "ditch_right", showright)
	Call("DLR:Activate", showright)
end

function Update(time)
    oldUpdate(time)
    local fullNumber = Call("GetRVNumber")
    local engineNumber = string.sub(fullNumber, 2, 4)
	-- pulls the entire number, then defines a substring that's just the numerical part

		Call("Childs_NEW:SetText", engineNumber, 0)
		Call("Childs_NEW:ActivateNode", "decals_NEW", 1)
		Call("Childs_OLD:ActivateNode", "decalflag", 0)
		-- sets a default state for the decals
		
		if string.find(fullNumber, ";N=1") then
			Call("Childs_OLD:SetText", "", 0)
			Call("Childs_OLD:ActivateNode", "decalflag", 0)
			Call("Childs_NEW:SetText", engineNumber, 0)
			Call("Childs_NEW:ActivateNode", "decals_NEW", 1)			
		end
		-- looks for a specific string in the entire number, and if it does then it shows and hides certain parts of the model

		if string.find(fullNumber, ";N=2") then
			Call("Childs_OLD:SetText", engineNumber, 0)
			Call("Childs_OLD:ActivateNode", "decalflag", 1)
			Call("Childs_NEW:SetText", "", 0)
			Call("Childs_NEW:ActivateNode", "decals_NEW", 0)
		end
		-- same as above
		
		
	Call("ActivateNode","primarydigits_4","0")
	Call("ActivateNode","decalflag","0")
	Call("ActivateNode","logo","0")

	SetDitchLights()
		
	local playerState = Call("GetIsPlayer")
	local trainSpeed = Call("GetSpeed")

	if playerState == 0 and trainSpeed >= 0.1 then
		Call( "FH1:Activate", 1 )
		Call( "FH2:Activate", 1 )
		Call( "BH1:Activate", 0 )
		Call( "BH2:Activate", 0 )
		Call("Childs_NEW:ActivateNode", "headlightcovers", 1)
	elseif playerState == 0 and trainSpeed < 0.1 then
		Call( "FH1:Activate", 0 )
		Call( "FH2:Activate", 0 )
		Call( "BH1:Activate", 0 )
		Call( "BH2:Activate", 0 )
		Call("Childs_NEW:ActivateNode", "headlightcovers", 0)
	end
end